diff -ur current/kaffe/libraries/clib/native/Math.c my-kaffe/libraries/clib/native/Math.c
--- current/kaffe/libraries/clib/native/Math.c	Mon Feb  8 06:07:18 1999
+++ my-kaffe/libraries/clib/native/Math.c	Fri Apr  5 01:23:39 2002
@@ -100,7 +100,7 @@
 }
 
 double
-java_lang_Math_pow(double v1, double v2)
+java_lang_Math_pow0(double v1, double v2)
 {
 	return (pow(v1, v2));
 }
diff -ur current/kaffe/libraries/clib/native/String.c my-kaffe/libraries/clib/native/String.c
--- current/kaffe/libraries/clib/native/String.c	Wed Jan 19 11:59:20 2000
+++ my-kaffe/libraries/clib/native/String.c	Fri Apr  5 01:09:14 2002
@@ -41,7 +41,11 @@
   unsigned char  bs[256];
   int *ibs;
 
-  if ( !pat || !str ) return -1;
+  if (pat == 0) {
+    SignalError("java.lang.NullPointerException", "");
+  }
+
+  if ( !str ) return -1;
 
   a = &(unhand_array(unhand(str)->value)->body[unhand(str)->offset]);
   n = unhand(str)->count;
diff -ur current/kaffe/libraries/javalib/java/lang/Math.java my-kaffe/libraries/javalib/java/lang/Math.java
--- current/kaffe/libraries/javalib/java/lang/Math.java	Mon Dec  6 22:08:18 1999
+++ my-kaffe/libraries/javalib/java/lang/Math.java	Fri Apr  5 01:24:18 2002
@@ -91,7 +91,14 @@
 
 public static long min(long a, long b) { return (a < b) ? (a) : (b); }
 
-native public static double pow(double a, double b);
+public static double pow(double a, double b) {
+	if (abs(a) == 1.0 && Double.isInfinite(b)) {
+		return Double.NaN;
+	}
+	return pow0(a, b);
+}
+
+native private static double pow0(double a, double b);
 
 public static synchronized double random() {
 	return MathRandom.random.nextDouble();
diff -ur current/kaffe/libraries/javalib/java/lang/StringBuffer.java my-kaffe/libraries/javalib/java/lang/StringBuffer.java
--- current/kaffe/libraries/javalib/java/lang/StringBuffer.java	Thu Nov 22 07:21:12 2001
+++ my-kaffe/libraries/javalib/java/lang/StringBuffer.java	Fri Apr  5 01:27:07 2002
@@ -97,9 +97,20 @@
 	return buffer[index];
 }
 
-public synchronized StringBuffer deleteCharAt(int index) {
-	if (index < 0 || index >= used)
+private void checkIndex(int index) {
+	if (index < 0 || index > length()) {
+		throw new StringIndexOutOfBoundsException();
+	}
+}
+
+private void checkExistingIndex(int index) {
+	if (index < 0 || index >= length()) {
 		throw new StringIndexOutOfBoundsException();
+	}
+}
+
+public synchronized StringBuffer deleteCharAt(int index) {
+	checkExistingIndex(index);
 	System.arraycopy(buffer, index + 1, buffer, index, --used - index);
 	return this;
 }
@@ -193,14 +204,13 @@
 }
 
 public StringBuffer insert(int offset, char[] str) {
+	checkIndex(offset);
 	return insert(offset, str, 0, str.length);
 }
 
 public synchronized StringBuffer insert(int index, char[] str,
 		int offset, int len) {
-	if (index < 0 || index > used) {
-		throw new StringIndexOutOfBoundsException();
-	}
+	checkIndex(index);
 	if (offset < 0 || len < 0 || offset + len > str.length) {
 		throw new StringIndexOutOfBoundsException();
 	}
@@ -250,8 +260,7 @@
 }
 
 public synchronized void setCharAt(int index, char ch) {
-	if (index < 0 || index >= used)
-		throw new StringIndexOutOfBoundsException();
+	checkIndex(index);
 	if (isStringized)				// optimization
 		ensureCapacity(used, true);
 	buffer[index] = ch;
